/***** interface.c *********************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 ***************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"
#include "stringUtil.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  char c;
  char *optString = "S:w:s:uhdp";

  args = (Args *)emalloc(sizeof(Args));

  args->s = NULL;
  args->D = DEFAULT_D;
  args->S = 0;
  args->h = 0;
  args->e = 1;
  args->d = 0;
  args->p = 0;
  args->u = 0;
  args->w = 0;
  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 's':                           /* sbjct file */
      args->s = optarg;
      args->e = 0;
      break;
    case 'S':                           /* step size */
      args->S = atoi(optarg);
      break;
    case 'w':                           /* window size */
      args->w = atoi(optarg);
      break;
    case 'p':                           /* print program information */
      args->p = 1;
      break;
    case 'u':                           /* print shulens */
      args->u = 1;
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'd':                           /* debug messages */
      args->d = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  if(!args->S)
    args->S = args->w/10;
  return args;
}


void printUsage(char *version){
  printf("pim version %s\n", version);
  printf("purpose: Estimate the number of mismatches/site (pi) from a pair of unaligned DNA sequences\n");
  printf("usage: pim -s sbjct.fasta [options] [queryFile(s)]\n");
  printf("options:\n");
  printf("\t-s <FILE> subject sequences in FASTA format read from FILE\n");
  printf("\t[-w <NUM> window length; default: no window analysis]\n");
  printf("\t[-S <NUM> steps between windows; default: NUM=window_length/10]\n");
  printf("\t[-u print shulens; default: print pim]\n");
  printf("\t[-d print debugging messages (best used in conjunction with grep)]\n");
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-p print information about program and exit]\n");			     
  exit(0);
}

void printSplash(char *version){
  printf("********************************************************\n");
  printf("*                  pim, version %s                    *\n", version);
  printf("*      Estimate pi_m from unaligned sequences          *\n");
  printf("*------------------------------------------------------*\n");
  printf("*                                                      *\n");  
  printf("* REFERENCE                                            *\n");
  printf("* Haubold, B., F. A. Reed, and P. Pfaffelhuber (2010). *\n");
  printf("* Alignment-free estimation of nucleotide diversity    *\n");
  printf("* identifies a novel genomic outlier of unusually low  *\n");
  printf("* genetic diversity in Drosophila melanogaster. In     *\n");
  printf("* preparation.                                         *\n");
  printf("*                                                      *\n");  
  printf("* CONTACT                                              *\n");
  printf("* Code maintained by Bernhard Haubold,                 *\n");
  printf("* haubold@evolbio.mpg.de                               *\n");
  printf("*                                                      *\n");
  printf("* ACKNOWLEDGEMENT                                      *\n");
  printf("* This software is based on the dss_sort library by G. *\n");
  printf("* Manzini; for further details see: G. Manzini, P.     *\n");
  printf("* Ferragina (2002). Engineering a lightweight          *\n");
  printf("* suffix array construction algorithm. Proc. 10th      *\n");
  printf("* European Symposium on Algorithms (ESA '02).          *\n");
  printf("* Springer Verlag Lecture Notes in Computer Science    *\n");
  printf("* n. 2461, pp 698-710.                                 *\n");
  printf("* http://www.mfn.unipmn.it/~manzini/lightweight        *\n");
  printf("*                                                      *\n");
  printf("* LICENSE                                              *\n");
  printf("* This software is distributed under the GNU General   *\n");
  printf("* Public License. You should have received a copy      *\n");
  printf("* of the licence together with this software. If       *\n");
  printf("* not, see http://www.gnu.org/licenses/.               *\n");
  printf("********************************************************\n");
  exit(0);
}
